@@ -87,7 +87,9 @@ class FakeServer(object):
             self, id, name, status, addresses=None,
             accessIPv4='', accessIPv6='', private_v4='',
             private_v6='', public_v4='', public_v6='',
-            flavor=None, image=None, adminPass=None):
+            interface_ip='',
+            flavor=None, image=None, adminPass=None,
+            metadata=None):
         self.id = id
         self.name = name
         self.status = status
@@ -108,6 +110,8 @@ class FakeServer(object):
         self.private_v6 = private_v6
         self.public_v6 = public_v6
         self.adminPass = adminPass
+        self.metadata = metadata
+        self.interface_ip = interface_ip
 
 
 class FakeService(object):
@@ -244,3 +248,27 @@ class FakeStack(object):
         self.stack_name = name
         self.stack_description = description
         self.stack_status = status
+
+
+class FakeZone(object):
+    def __init__(self, id, name, type_, email, description,
+                 ttl, masters):
+        self.id = id
+        self.name = name
+        self.type_ = type_
+        self.email = email
+        self.description = description
+        self.ttl = ttl
+        self.masters = masters
+
+
+class FakeRecordset(object):
+    def __init__(self, zone, id, name, type_, description,
+                 ttl, records):
+        self.zone = zone
+        self.id = id
+        self.name = name
+        self.type_ = type_
+        self.description = description
+        self.ttl = ttl
+        self.records = records
