@@ -19,6 +19,7 @@ test_floating_ip_neutron
 Tests Floating IP resource methods for Neutron
 """
 
+import mock
 from mock import patch
 import os_client_config
 
@@ -340,31 +341,34 @@ class TestFloatingIP(base.TestCase):
             mock_keystone_session,
             mock_nova_client):
         mock_has_service.return_value = True
-        mock__neutron_create_floating_ip.return_value = \
-            self.mock_floating_ip_list_rep['floatingips'][0]
+        fip = _utils.normalize_neutron_floating_ips(
+            self.mock_floating_ip_list_rep['floatingips'])[0]
+        mock__neutron_create_floating_ip.return_value = fip
         mock_keystone_session.get_project_id.return_value = \
             '4969c491a3c74ee4af974e6d800c62df'
+        fake_server = meta.obj_to_dict(fakes.FakeServer('1234', '', 'ACTIVE'))
 
         self.client.add_ips_to_server(
-            dict(id='1234'), ip_pool='my-network', reuse=False)
+            fake_server, ip_pool='my-network', reuse=False)
 
         mock__neutron_create_floating_ip.assert_called_once_with(
-            network_name_or_id='my-network', server=None)
+            network_name_or_id='my-network', server=None,
+            fixed_address=None, nat_destination=None, wait=False, timeout=60)
         mock_attach_ip_to_server.assert_called_once_with(
-            server={'id': '1234'}, fixed_address=None,
-            floating_ip=self.floating_ip, wait=False, timeout=60)
+            server=fake_server, fixed_address=None,
+            floating_ip=fip, wait=False, timeout=mock.ANY)
 
     @patch.object(OpenStackCloud, 'keystone_session')
     @patch.object(OpenStackCloud, '_neutron_create_floating_ip')
     @patch.object(OpenStackCloud, '_neutron_list_floating_ips')
-    @patch.object(OpenStackCloud, 'search_networks')
+    @patch.object(OpenStackCloud, 'list_networks')
     @patch.object(OpenStackCloud, 'has_service')
     def test_available_floating_ip_new(
-            self, mock_has_service, mock_search_networks,
+            self, mock_has_service, mock_list_networks,
             mock__neutron_list_floating_ips,
             mock__neutron_create_floating_ip, mock_keystone_session):
         mock_has_service.return_value = True
-        mock_search_networks.return_value = [self.mock_get_network_rep]
+        mock_list_networks.return_value = [self.mock_get_network_rep]
         mock__neutron_list_floating_ips.return_value = []
         mock__neutron_create_floating_ip.return_value = \
             self.mock_floating_ip_new_rep['floatingip']
@@ -383,19 +387,49 @@ class TestFloatingIP(base.TestCase):
     def test_delete_floating_ip_existing(
             self, mock_has_service, mock_neutron_client, mock_get_floating_ip):
         mock_has_service.return_value = True
-        mock_get_floating_ip.return_value = {
+        fake_fip = {
             'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            'floating_ip_address': '172.99.106.167',
         }
+
+        mock_get_floating_ip.side_effect = [fake_fip, fake_fip, None]
         mock_neutron_client.delete_floatingip.return_value = None
 
         ret = self.client.delete_floating_ip(
-            floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7')
+            floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            retry=2)
 
         mock_neutron_client.delete_floatingip.assert_called_with(
             floatingip='2f245a7b-796b-4f26-9cf9-9e82d248fda7'
         )
+        self.assertEqual(mock_get_floating_ip.call_count, 3)
         self.assertTrue(ret)
 
+    @patch.object(OpenStackCloud, 'get_floating_ip')
+    @patch.object(OpenStackCloud, 'neutron_client')
+    @patch.object(OpenStackCloud, 'has_service')
+    def test_delete_floating_ip_existing_no_delete(
+            self, mock_has_service, mock_neutron_client, mock_get_floating_ip):
+        mock_has_service.return_value = True
+        fake_fip = {
+            'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            'floating_ip_address': '172.99.106.167',
+        }
+
+        mock_get_floating_ip.side_effect = [fake_fip, fake_fip, fake_fip]
+        mock_neutron_client.delete_floatingip.return_value = None
+
+        self.assertRaises(
+            exc.OpenStackCloudException,
+            self.client.delete_floating_ip,
+            floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            retry=2)
+
+        mock_neutron_client.delete_floatingip.assert_called_with(
+            floatingip='2f245a7b-796b-4f26-9cf9-9e82d248fda7'
+        )
+        self.assertEqual(mock_get_floating_ip.call_count, 3)
+
     @patch.object(OpenStackCloud, 'neutron_client')
     @patch.object(OpenStackCloud, 'has_service')
     def test_delete_floating_ip_not_found(
@@ -436,6 +470,28 @@ class TestFloatingIP(base.TestCase):
             }
         )
 
+    @patch.object(OpenStackCloud, 'delete_floating_ip')
+    @patch.object(OpenStackCloud, 'get_server_by_id')
+    @patch.object(OpenStackCloud, 'create_floating_ip')
+    @patch.object(OpenStackCloud, 'has_service')
+    def test_add_ip_refresh_timeout(
+            self, mock_has_service, mock_create_floating_ip,
+            mock_get_server_by_id, mock_delete_floating_ip):
+        mock_has_service.return_value = True
+
+        mock_create_floating_ip.return_value = self.floating_ip
+        mock_get_server_by_id.return_value = self.fake_server
+
+        self.assertRaises(
+            exc.OpenStackCloudTimeout,
+            self.client._add_auto_ip,
+            server=self.fake_server,
+            wait=True, timeout=2,
+            reuse=False)
+
+        mock_delete_floating_ip.assert_called_once_with(
+            self.floating_ip['id'])
+
     @patch.object(OpenStackCloud, 'get_floating_ip')
     @patch.object(OpenStackCloud, 'neutron_client')
     @patch.object(OpenStackCloud, 'has_service')
@@ -478,3 +534,52 @@ class TestFloatingIP(base.TestCase):
             fixed_address='192.0.2.129')
 
         self.assertEqual(server, self.fake_server)
+
+    @patch.object(OpenStackCloud, 'delete_floating_ip')
+    @patch.object(OpenStackCloud, 'list_floating_ips')
+    @patch.object(OpenStackCloud, '_use_neutron_floating')
+    def test_cleanup_floating_ips(
+            self, mock_use_neutron_floating, mock_list_floating_ips,
+            mock_delete_floating_ip):
+        mock_use_neutron_floating.return_value = True
+        floating_ips = [{
+            "id": "this-is-a-floating-ip-id",
+            "fixed_ip_address": None,
+            "internal_network": None,
+            "floating_ip_address": "203.0.113.29",
+            "network": "this-is-a-net-or-pool-id",
+            "attached": False,
+            "status": "ACTIVE"
+        }, {
+            "id": "this-is-an-attached-floating-ip-id",
+            "fixed_ip_address": None,
+            "internal_network": None,
+            "floating_ip_address": "203.0.113.29",
+            "network": "this-is-a-net-or-pool-id",
+            "attached": True,
+            "status": "ACTIVE"
+        }]
+
+        mock_list_floating_ips.return_value = floating_ips
+
+        self.client.delete_unattached_floating_ips()
+
+        mock_delete_floating_ip.assert_called_once_with(
+            id="this-is-a-floating-ip-id",
+            timeout=60, wait=False)
+
+    @patch.object(OpenStackCloud, '_submit_create_fip')
+    @patch.object(OpenStackCloud, '_get_free_fixed_port')
+    @patch.object(OpenStackCloud, 'get_external_networks')
+    def test_create_floating_ip_no_port(
+            self, mock_get_ext_nets, mock_get_free_fixed_port,
+            mock_submit_create_fip):
+        fake_port = dict(id='port-id')
+        mock_get_ext_nets.return_value = [self.mock_get_network_rep]
+        mock_get_free_fixed_port.return_value = (fake_port, '10.0.0.2')
+        mock_submit_create_fip.return_value = dict(port_id=None)
+
+        self.assertRaises(
+            exc.OpenStackCloudException,
+            self.client._neutron_create_floating_ip,
+            server=dict(id='some-server'))
