@@ -64,20 +64,48 @@ class FakeCloud(object):
     def list_server_security_groups(self, server):
         return []
 
+    def get_default_network(self):
+        return None
 
-class FakeServer(object):
-    id = 'test-id-0'
-    metadata = {'group': 'test-group'}
-    addresses = {'private': [{'OS-EXT-IPS:type': 'fixed',
-                              'addr': PRIVATE_V4,
-                              'version': 4}],
-                 'public': [{'OS-EXT-IPS:type': 'floating',
-                             'addr': PUBLIC_V4,
-                             'version': 4}]}
-    flavor = {'id': '101'}
-    image = {'id': '471c2475-da2f-47ac-aba5-cb4aa3d546f5'}
-    accessIPv4 = ''
-    accessIPv6 = ''
+standard_fake_server = fakes.FakeServer(
+    id='test-id-0',
+    name='test-id-0',
+    status='ACTIVE',
+    metadata={'group': 'test-group'},
+    addresses={'private': [{'OS-EXT-IPS:type': 'fixed',
+                            'addr': PRIVATE_V4,
+                            'version': 4}],
+               'public': [{'OS-EXT-IPS:type': 'floating',
+                           'addr': PUBLIC_V4,
+                           'version': 4}]},
+    flavor={'id': '101'},
+    image={'id': '471c2475-da2f-47ac-aba5-cb4aa3d546f5'},
+    accessIPv4='',
+    accessIPv6='',
+)
+
+SUBNETS_WITH_NAT = [
+    {
+        u'name': u'',
+        u'enable_dhcp': True,
+        u'network_id': u'5ef0358f-9403-4f7b-9151-376ca112abf7',
+        u'tenant_id': u'29c79f394b2946f1a0f8446d715dc301',
+        u'dns_nameservers': [],
+        u'ipv6_ra_mode': None,
+        u'allocation_pools': [
+            {
+                u'start': u'10.10.10.2',
+                u'end': u'10.10.10.254'
+            }
+        ],
+        u'gateway_ip': u'10.10.10.1',
+        u'ipv6_address_mode': None,
+        u'ip_version': 4,
+        u'host_routes': [],
+        u'cidr': u'10.10.10.0/24',
+        u'id': u'14025a85-436e-4418-b0ee-f5b12a50f9b4'
+    },
+]
 
 
 class TestMeta(base.TestCase):
@@ -121,18 +149,20 @@ class TestMeta(base.TestCase):
             addrs, key_name='public', ext_tag='fixed', version=6))
 
     def test_get_server_ip(self):
-        srv = meta.obj_to_dict(FakeServer())
+        srv = meta.obj_to_dict(standard_fake_server)
         self.assertEqual(
-            PRIVATE_V4, meta.get_server_private_ip(srv, self.cloud))
+            PRIVATE_V4, meta.get_server_ip(srv, ext_tag='fixed'))
         self.assertEqual(
-            PUBLIC_V4, meta.get_server_external_ipv4(self.cloud, srv))
+            PUBLIC_V4, meta.get_server_ip(srv, ext_tag='floating'))
 
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
-    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
     def test_get_server_private_ip(
-            self, mock_search_networks, mock_has_service):
+            self, mock_list_networks, mock_list_subnets, mock_has_service):
         mock_has_service.return_value = True
-        mock_search_networks.return_value = [{
+        mock_list_subnets.return_value = SUBNETS_WITH_NAT
+        mock_list_networks.return_value = [{
             'id': 'test-net-id',
             'name': 'test-net-name'
         }]
@@ -150,29 +180,31 @@ class TestMeta(base.TestCase):
         self.assertEqual(
             PRIVATE_V4, meta.get_server_private_ip(srv, self.cloud))
         mock_has_service.assert_called_with('network')
-        mock_search_networks.assert_called_with(
-            filters={'router:external': False}
-        )
+        mock_list_networks.assert_called_once_with()
 
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
     @mock.patch.object(shade.OpenStackCloud, 'list_server_security_groups')
     @mock.patch.object(shade.OpenStackCloud, 'get_volumes')
     @mock.patch.object(shade.OpenStackCloud, 'get_image_name')
     @mock.patch.object(shade.OpenStackCloud, 'get_flavor_name')
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
-    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
     def test_get_server_private_ip_devstack(
-            self, mock_search_networks, mock_has_service,
+            self, mock_list_networks, mock_has_service,
             mock_get_flavor_name, mock_get_image_name,
             mock_get_volumes,
-            mock_list_server_security_groups):
+            mock_list_server_security_groups,
+            mock_list_subnets):
         mock_get_image_name.return_value = 'cirros-0.3.4-x86_64-uec'
         mock_get_flavor_name.return_value = 'm1.tiny'
         mock_has_service.return_value = True
         mock_get_volumes.return_value = []
-        mock_search_networks.return_value = [
+        mock_list_subnets.return_value = SUBNETS_WITH_NAT
+        mock_list_networks.return_value = [
             {
                 'id': 'test_pnztt_net',
-                'name': 'test_pnztt_net'
+                'name': 'test_pnztt_net',
+                'router:external': False,
             },
             {
                 'id': 'private',
@@ -197,20 +229,21 @@ class TestMeta(base.TestCase):
 
         self.assertEqual(PRIVATE_V4, srv['private_v4'])
         mock_has_service.assert_called_with('volume')
-        mock_search_networks.assert_called_with(
-            filters={'router:external': False}
-        )
+        mock_list_networks.assert_called_once_with()
 
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
-    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
     def test_get_server_external_ipv4_neutron(
-            self, mock_search_networks,
+            self, mock_list_networks, mock_list_subnets,
             mock_has_service):
         # Testing Clouds with Neutron
         mock_has_service.return_value = True
-        mock_search_networks.return_value = [{
+        mock_list_subnets.return_value = []
+        mock_list_networks.return_value = [{
             'id': 'test-net-id',
-            'name': 'test-net'
+            'name': 'test-net',
+            'router:external': True,
         }]
 
         srv = meta.obj_to_dict(fakes.FakeServer(
@@ -223,6 +256,56 @@ class TestMeta(base.TestCase):
 
         self.assertEqual(PUBLIC_V4, ip)
 
+    @mock.patch.object(shade.OpenStackCloud, 'has_service')
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
+    def test_get_server_external_provider_ipv4_neutron(
+            self, mock_list_networks, mock_list_subnets,
+            mock_has_service):
+        # Testing Clouds with Neutron
+        mock_has_service.return_value = True
+        mock_list_subnets.return_value = SUBNETS_WITH_NAT
+        mock_list_networks.return_value = [{
+            'id': 'test-net-id',
+            'name': 'test-net',
+            'provider:network_type': 'vlan',
+        }]
+
+        srv = meta.obj_to_dict(fakes.FakeServer(
+            id='test-id', name='test-name', status='ACTIVE',
+            addresses={'test-net': [{
+                'addr': PUBLIC_V4,
+                'version': 4}]},
+        ))
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
+
+        self.assertEqual(PUBLIC_V4, ip)
+
+    @mock.patch.object(shade.OpenStackCloud, 'has_service')
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
+    def test_get_server_external_none_ipv4_neutron(
+            self, mock_list_networks, mock_list_subnets,
+            mock_has_service):
+        # Testing Clouds with Neutron
+        mock_has_service.return_value = True
+        mock_list_subnets.return_value = SUBNETS_WITH_NAT
+        mock_list_networks.return_value = [{
+            'id': 'test-net-id',
+            'name': 'test-net',
+            'router:external': False,
+        }]
+
+        srv = meta.obj_to_dict(fakes.FakeServer(
+            id='test-id', name='test-name', status='ACTIVE',
+            addresses={'test-net': [{
+                'addr': PUBLIC_V4,
+                'version': 4}]},
+        ))
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
+
+        self.assertEqual(None, ip)
+
     def test_get_server_external_ipv4_neutron_accessIPv4(self):
         srv = meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE',
@@ -240,16 +323,18 @@ class TestMeta(base.TestCase):
         self.assertEqual(PUBLIC_V6, ip)
 
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
-    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    @mock.patch.object(shade.OpenStackCloud, 'list_subnets')
+    @mock.patch.object(shade.OpenStackCloud, 'list_networks')
     @mock.patch.object(shade.OpenStackCloud, 'search_ports')
     @mock.patch.object(meta, 'get_server_ip')
     def test_get_server_external_ipv4_neutron_exception(
             self, mock_get_server_ip, mock_search_ports,
-            mock_search_networks,
+            mock_list_networks, mock_list_subnets,
             mock_has_service):
         # Testing Clouds with a non working Neutron
         mock_has_service.return_value = True
-        mock_search_networks.return_value = []
+        mock_list_subnets.return_value = []
+        mock_list_networks.return_value = []
         mock_search_ports.side_effect = neutron_exceptions.NotFound()
         mock_get_server_ip.return_value = PUBLIC_V4
 
@@ -317,7 +402,11 @@ class TestMeta(base.TestCase):
              'test-region_test-az',
              'test-name_test-region_test-az'],
             meta.get_groups_from_server(
-                FakeCloud(), meta.obj_to_dict(FakeServer()), server_vars))
+                FakeCloud(),
+                meta.obj_to_dict(standard_fake_server),
+                server_vars
+            )
+        )
 
     def test_obj_list_to_dict(self):
         """Test conversion of a list of objects to a list of dictonaries"""
@@ -342,7 +431,7 @@ class TestMeta(base.TestCase):
         mock_list_server_security_groups.return_value = [
             {'name': 'testgroup', 'id': '1'}]
 
-        server = meta.obj_to_dict(FakeServer())
+        server = meta.obj_to_dict(standard_fake_server)
         hostvars = meta.get_hostvars_from_server(FakeCloud(), server)
 
         mock_list_server_security_groups.assert_called_once_with(server)
@@ -359,7 +448,7 @@ class TestMeta(base.TestCase):
 
         hostvars = meta.get_hostvars_from_server(
             FakeCloud(), _utils.normalize_server(
-                meta.obj_to_dict(FakeServer()),
+                meta.obj_to_dict(standard_fake_server),
                 cloud_name='CLOUD_NAME',
                 region_name='REGION_NAME'))
         self.assertNotIn('links', hostvars)
@@ -370,9 +459,11 @@ class TestMeta(base.TestCase):
         self.assertEquals('REGION_NAME', hostvars['region'])
         self.assertEquals('CLOUD_NAME', hostvars['cloud'])
         self.assertEquals("test-image-name", hostvars['image']['name'])
-        self.assertEquals(FakeServer.image['id'], hostvars['image']['id'])
+        self.assertEquals(standard_fake_server.image['id'],
+                          hostvars['image']['id'])
         self.assertNotIn('links', hostvars['image'])
-        self.assertEquals(FakeServer.flavor['id'], hostvars['flavor']['id'])
+        self.assertEquals(standard_fake_server.flavor['id'],
+                          hostvars['flavor']['id'])
         self.assertEquals("test-flavor-name", hostvars['flavor']['name'])
         self.assertNotIn('links', hostvars['flavor'])
         # test having volumes
@@ -390,7 +481,7 @@ class TestMeta(base.TestCase):
         fake_cloud = FakeCloud()
         fake_cloud.force_ipv4 = True
         hostvars = meta.get_hostvars_from_server(
-            fake_cloud, meta.obj_to_dict(FakeServer()))
+            fake_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEqual(PUBLIC_V4, hostvars['interface_ip'])
 
     @mock.patch.object(shade.meta, 'get_server_external_ipv4')
@@ -400,21 +491,21 @@ class TestMeta(base.TestCase):
         cloud = FakeCloud()
         cloud.private = True
         hostvars = meta.get_hostvars_from_server(
-            cloud, meta.obj_to_dict(FakeServer()))
+            cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEqual(PRIVATE_V4, hostvars['interface_ip'])
 
     @mock.patch.object(shade.meta, 'get_server_external_ipv4')
     def test_image_string(self, mock_get_server_external_ipv4):
         mock_get_server_external_ipv4.return_value = PUBLIC_V4
 
-        server = FakeServer()
+        server = standard_fake_server
         server.image = 'fake-image-id'
         hostvars = meta.get_hostvars_from_server(
             FakeCloud(), meta.obj_to_dict(server))
         self.assertEquals('fake-image-id', hostvars['image']['id'])
 
     def test_az(self):
-        server = FakeServer()
+        server = standard_fake_server
         server.__dict__['OS-EXT-AZ:availability_zone'] = 'az1'
 
         hostvars = _utils.normalize_server(
@@ -433,7 +524,7 @@ class TestMeta(base.TestCase):
         fake_volume_dict = meta.obj_to_dict(fake_volume)
         mock_cloud.get_volumes.return_value = [fake_volume_dict]
         hostvars = meta.get_hostvars_from_server(
-            mock_cloud, meta.obj_to_dict(FakeServer()))
+            mock_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEquals('volume1', hostvars['volumes'][0]['id'])
         self.assertEquals('/dev/sda0', hostvars['volumes'][0]['device'])
 
@@ -441,7 +532,7 @@ class TestMeta(base.TestCase):
         fake_cloud = FakeCloud()
         fake_cloud.service_val = False
         hostvars = meta.get_hostvars_from_server(
-            fake_cloud, meta.obj_to_dict(FakeServer()))
+            fake_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEquals([], hostvars['volumes'])
 
     def test_unknown_volume_exception(self):
@@ -457,11 +548,11 @@ class TestMeta(base.TestCase):
             FakeException,
             meta.get_hostvars_from_server,
             mock_cloud,
-            meta.obj_to_dict(FakeServer()))
+            meta.obj_to_dict(standard_fake_server))
 
     def test_obj_to_dict(self):
         cloud = FakeCloud()
-        cloud.server = FakeServer()
+        cloud.server = standard_fake_server
         cloud_dict = meta.obj_to_dict(cloud)
         self.assertEqual(FakeCloud.name, cloud_dict['name'])
         self.assertNotIn('_unused', cloud_dict)
