@@ -67,6 +67,31 @@ class TestUsers(base.TestCase):
         self.assertEqual(name, user.name)
         self.assertEqual(email, user.email)
 
+    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
+    def test_update_user_password_v2(self, mock_keystone, mock_api_version):
+        mock_api_version.return_value = '2'
+        name = 'Mickey Mouse'
+        email = 'mickey@disney.com'
+        password = 'mice-rule'
+        domain_id = '1'
+        user = {'id': '1', 'name': name, 'email': email}
+        fake_user = fakes.FakeUser(**user)
+        munch_fake_user = munch.Munch(user)
+        mock_keystone.users.list.return_value = [fake_user]
+        mock_keystone.users.get.return_value = fake_user
+        mock_keystone.users.update.return_value = fake_user
+        mock_keystone.users.update_password.return_value = fake_user
+        user = self.cloud.update_user(name, name=name, email=email,
+                                      password=password,
+                                      domain_id=domain_id)
+        mock_keystone.users.update.assert_called_once_with(
+            user=munch_fake_user, name=name, email=email)
+        mock_keystone.users.update_password.assert_called_once_with(
+            user=munch_fake_user, password=password)
+        self.assertEqual(name, user.name)
+        self.assertEqual(email, user.email)
+
     @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
     @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
     def test_create_user_v3_no_domain(self, mock_keystone, mock_api_version):
@@ -76,7 +101,8 @@ class TestUsers(base.TestCase):
         password = 'mice-rule'
         with testtools.ExpectedException(
                 shade.OpenStackCloudException,
-                "User creation requires an explicit domain_id argument."
+                "User or project creation requires an explicit"
+                " domain_id argument."
         ):
             self.cloud.create_user(name=name, email=email, password=password)
 
